### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""SiteUrl adapters for the Zope 3 based pager package

$Id: pagedsourceadapter.py 1440 2007-09-29 15:24:49Z anatoly $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 1440 $"
__date__ = "$Date: 2007-09-29 18:24:49 +0300 (Сб, 29 сен 2007) $"

from zope.interface import Interface, implements
from zope.cachedescriptors.property import CachedProperty
from ks.pager.interfaces import IPagedSource
from zope.component import adapts

class PagedSourceAdapter(object) :

    adapts(Interface)

    implements(IPagedSource)

    def __init__(self, context):
        self.context = context

    def getChunk(self, start, chunkSize, *kv, **kw):
        """See IPagedSource interface"""

        if self.context is None:
            return []
        if self.getCount():
            return self.context[start:start+chunkSize]
        return []

    def getCount(self, *kv, **kw):
        """See IPagedSource interface"""

        return len(list(self.context))



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""SiteUrl adapters for the Zope 3 based pager package

$Id: pageradapter.py 1408 2007-09-18 14:56:18Z anatoly $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 1408 $"
__date__ = "$Date: 2007-09-18 17:56:18 +0300 (Вт, 18 сен 2007) $"

from zope.interface import Interface, implements
from zope.cachedescriptors.property import CachedProperty
from zope.publisher.browser import BrowserView
from ks.pager.interfaces import IPagerParams, IPager, IPagedSource, IChunk
from zope.component import adapts
from zope.schema.fieldproperty import FieldProperty
from zope.publisher.interfaces.http import IHTTPRequest
import math
import re
import urlparse, urllib
import cgi

class Chunk(object):

    implements(IChunk)

    title = FieldProperty(IChunk['title'])

    url = FieldProperty(IChunk['url'])

    selected = FieldProperty(IChunk['selected'])

    def __init__(self, title=None, url=None, selected=None):
        self.title = title
        self.url = url
        self.selected = selected

class PagerAdapter(object) :

    adapts(IPagedSource, IPagerParams, IHTTPRequest)

    implements(IPager)

    def __init__(self, context, params, request):
        self.context = context
        self.params = params
        self.request = request

    def init(self, *kv, **kw):
        if 'start' in kw:
            del kw['start']
        if 'chunkSize' in kw:
            del kw['chunkSize']
        self.kv = kv
        self.kw = kw

    @CachedProperty
    def nextChunkUrl(self):
        """Get Next Chunk Url"""
        return self.getChunkUrl(False, self.start)

    @CachedProperty
    def prevChunkUrl(self):
        """Get Previous Chunk Url"""
        return self.getChunkUrl(True, self.start)

    @CachedProperty
    def haveNextChunk(self):
        """Have Next Chunk Url Flag"""
        return (self.start + self.chunkSize) < self.count

    @CachedProperty
    def havePrevChunk(self):
        """Have Previous Chunk Url Flag"""
        return (self.start - self.chunkSize) >= 0

    @CachedProperty
    def chunkUrlList(self):
        """Get Chunk Url List"""
        chunkListCount = self.params.chunkCount
        chunkCount = self.chunkCount
        currentChunk = self.currentChunk
        chunkSize = self.chunkSize
        count = self.count
        if chunkCount < chunkListCount:
            chunkListCount = chunkCount
        stop = (currentChunk/chunkListCount + 1)*chunkListCount
        if stop > chunkCount:
            stop = chunkCount
        start = (currentChunk/chunkListCount)*int(chunkListCount)
        return [Chunk(title=unicode(x + 1),
                      selected=(x == currentChunk),
                      url=self.getChunkUrl(False, chunkSize*(x-1))
                      )
             for x in xrange(start, stop)]


    @CachedProperty
    def firstChunkUrl(self):
        """First Chunk Url"""
        return self.getChunkUrl(True, self.chunkSize)

    @CachedProperty
    def onFirstChunk(self):
        """On First Chunk Flag"""
        return self.currentChunk == 0

    @CachedProperty
    def lastChunkUrl(self):
        """Last Chunk Url"""
        return self.getChunkUrl(False, self.chunkSize * (self.chunkCount - 2))

    @CachedProperty
    def onLastChunk(self):
        """On Last Chunk Flag"""
        return self.currentChunk == (self.chunkCount -1)

    @property
    def chunk(self):
        """Get Chunk"""
        if (self.context is None) or (not self.count):
            return []

        return self.context.getChunk(self.start,
                                     self.chunkSize,
                                     *self.kv, **self.kw)

    @CachedProperty
    def count(self):
        """Get Count"""
        return self.context.getCount(*self.kv, **self.kw)

    @CachedProperty
    def chunkCount(self):
        """Get Chunk Count"""
        if self.count == 0:
            return 0
        return int(math.ceil(float(self.count)/self.chunkSize))

    def getChunkUrl(self, previous, start):
        """Get Chunk Url"""
        startKey = self.params.startKey
        count = self.count
        chunkSize = self.chunkSize
        chunkSizeKey = self.params.chunkSizeKey
        if previous:
            start -= chunkSize
        else:
            start += chunkSize
        if self.params.objectURL is not None:
            reqs = self.params.objectURL
        else:
            reqs = str(self.request.URL)
        parsed_query = list(urlparse.urlparse(reqs))
        params = cgi.parse_qsl(self.request['QUERY_STRING']) # не очень
        for key, value in params:
            if key == startKey:
                params.remove((key, value))
        params.append((startKey, start))
        parsed_query[-2] = urllib.urlencode(params)
        return urlparse.urlunparse(parsed_query)

    @CachedProperty
    def currentChunk(self):
        """Current Chunk Number"""
        start = self.start
        chunkSize = self.chunkSize
        count = self.count
        return int(math.ceil(float(start) / chunkSize))

    @CachedProperty
    def start(self):
        start = self.params.start
        count = self.count
        if start > count:
            return self.chunkSize * (self.chunkCount - 1)
        elif start < self.params.defaultStart:
            return self.params.defaultStart
        return start

    @CachedProperty
    def chunkSize(self):
        count = self.count
        chunkSize = self.params.chunkSize
        if chunkSize > count:
            if count > 0:
                return count
            return 1
        elif chunkSize < 1:
            return self.params.defaultChunkSize
        return chunkSize



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/) 
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)                             
# All right reserved, 2006-2007
#######################################################################
# Licensed under the Zope Public License, Version 2.1 (the "License"); you
# may not use this file except in compliance with the License. A copy of the
# License should accompany this distribution.
#
# This software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#######################################################################
# $Id: __init__.py 23735 2007-11-08 18:22:32Z anatoly $
#######################################################################

__import__('pkg_resources').declare_namespace(__name__)


### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""SiteUrl adapters for the Zope 3 based issue package

$Id: pagerparamsadapter.py 1408 2007-09-18 14:56:18Z anatoly $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 1408 $"
__date__ = "$Date: 2007-09-18 17:56:18 +0300 (Вт, 18 сен 2007) $"

from zope.interface import Interface, implements
from zope.publisher.browser import BrowserView
from ks.pager.interfaces import IPagerParams
from zope.component import adapts
from zope.publisher.interfaces.http import IHTTPRequest
from zope.schema.fieldproperty import FieldProperty

class PagerParamsAdapterBase(object):

    startKey = FieldProperty(IPagerParams['startKey'])

    defaultStart = FieldProperty(IPagerParams['defaultStart'])

    chunkSizeKey = FieldProperty(IPagerParams['chunkSizeKey'])

    defaultChunkSize = FieldProperty(IPagerParams['defaultChunkSize'])

    chunkCountKey = FieldProperty(IPagerParams['chunkCountKey'])

    defaultChunkCount = FieldProperty(IPagerParams['defaultChunkCount'])

    objectURL = FieldProperty(IPagerParams['objectURL'])

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def start(self):
        """See IPagerParams interface"""

        return int(self.request.form.get(self.startKey, self.defaultStart))

    @property
    def chunkSize(self):
        """See IPagerParams interface"""

        return int(self.request.form.get(self.chunkSizeKey, self.defaultChunkSize))

    @property
    def chunkCount(self):
        """See IPagerParams interface"""

        return int(self.request.form.get(self.chunkCountKey, self.defaultChunkCount))

class PagerParamsAdapter(PagerParamsAdapterBase):

    adapts(Interface, IHTTPRequest)

    implements(IPagerParams)



### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""SiteUrl adapters for the Zope 3 based issue package

$Id: pagerfactoryadapter.py 1270 2007-08-17 17:01:42Z anton $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "ZPL"
__version__ = "$Revision: 1270 $"
__date__ = "$Date: 2007-08-17 20:01:42 +0300 (Пт, 17 авг 2007) $"

from zope.interface import implements
from zope.publisher.browser import BrowserView
from ks.pager.interfaces import IPagerFactory, IPagerParams, IPager, IPagedSource
from zope.component import getMultiAdapter


class PagerFactoryAdapter(BrowserView):
    """Pager Factory View"""

    implements(IPagerFactory)

    def __call__(self):
        """See IPagerFactory interface"""

        return self

    def init(self, iterable, *kv, **kw):
        """See IPagerFactory interface"""

        params = getMultiAdapter([self.context, self.request],
                               interface=IPagerParams,
                               context=self.context)

        paged_iterable = IPagedSource(iterable)

        res = getMultiAdapter([paged_iterable, params, self.request],
                               interface=IPager,
                               context=self.context)
        res.init(*kv, **kw)
        return res


### -*- coding: utf-8 -*- #############################################
# Разработано компанией Ключевые Решения (http://keysolutions.ru/)
# Все права защищены, 2006-2007
#
# Developed by Key Solutions (http://keysolutions.ru/)
# All right reserved, 2006-2007
#######################################################################
"""Inrefaces for the Zope 3 based pager package

$Id: interfaces.py 12510 2007-10-30 13:53:20Z anatoly $
"""
__author__  = "Anatoly Bubenkov"
__license__ = "GPL"
__version__ = "$Revision: 12510 $"
__date__ = "$Date: 2007-10-30 15:53:20 +0200 (Вт, 30 окт 2007) $"

from zope.interface import Interface
from zope.schema import Int, URI, Bool, Tuple, Object, TextLine, ASCIILine

import zope.i18nmessageid
_ = zope.i18nmessageid.MessageFactory('ks.pager')

class IPagerFactory(Interface):
    """Pager Factory"""

    def init(self, iterable):
        """Get Adapted Iterable"""

class IPagerParams(Interface):
    """Pager Params"""

    start = Int(title=_(u'Start Element'))

    defaultStart = Int(title=_(u'Default Start Element'),
                       default=0)

    startKey = ASCIILine(title=_(u'Start Key'),
                         default='start')

    chunkSize = Int(title=_(u'Chunk Size'))

    defaultChunkSize = Int(title=_(u'Default Chunk Size'),
                           default = 4)

    chunkSizeKey = ASCIILine(title=_(u'Chunk Size Key'),
                             default='cnt')

    chunkCount = Int(title=_(u'Chunk Count on Page'))

    defaultChunkCount = Int(title=_(u'Default Chunk Count on Page'),
                            default = 10)

    chunkCountKey = ASCIILine(title=_(u'Chunk Count Key'),
                              default='chnkcnt')

    objectURL = ASCIILine(title=_(u'Object URL for Chunk URLs'),
                         )

class IChunk(Interface):

    title = TextLine(title=_(u'Title'),
                     required=False)

    url = URI(title=_(u'Url'))

    selected = Bool(title=_(u'Selected'),
                    required=False)

class IPagedSource(Interface):

    def getCount(*kv, **kw):
        """Get Full Element List Count"""

    def getChunk(start, chunkSize, *kv, **kw):
        """Get Chunk of Data"""

class IPager(Interface):

    nextChunkUrl = URI(title=_(u'Next Chunk URL'))

    prevChunkUrl = URI(title=_(u'Next Chunk URL'))

    haveNextChunk = Bool(title=_(u'Have Next Chunk'))

    havePrevChunk = Bool(title=_(u'Have Previous Chunk'))

    chunkUrlList = Tuple(title=_(u'Chunk URL List'),
                         value_type=Object(title=_(u'Chunk URL'),
                                           schema=IChunk)
                         )

    chunk = Tuple(title=_(u'Chunk Element List'),
                  value_type=Object(schema=IChunk)
                  )

    count = Int(title=_(u'Full Element List Count'))

    currentChunk = Int(title=_(u'Current Chunk Number'))

    firstChunkUrl = URI(title=_(u'First Chunk URL'))

    onFirstChunk = Bool(title=_(u'On First Chunk Flag'))

    lastChunkUrl = URI(title=_(u'Last Chunk URL'))

    onLastChunk = Bool(title=_(u'On Last Chunk Flag'))

    def init(*kv, **kw):
        """Init Pager"""



"""Setup for ks.pager package

$Id: setup.py 23736 2007-11-08 18:28:21Z anatoly $
"""

import os

from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('README.txt')
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    'Detailed Documentation\n'
    '**********************\n'
    + '\n' +
    read('src', 'ks', 'pager', 'README.txt')
    )

setup(
    name = 'ks.pager',
    version = '1.0.1',
    url = 'http://code.keysolutions.ru/ks.pager',
    license = 'ZPL 2.1',
    description = 'Easy get paged listings of objects with ks.formkeeper for Hivurt',
    author = 'Key Solutions Development TEAM',
    author_email = 'ksd-team@keysolutions.ru',
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development',
        'Framework :: Zope3',
        ],
    long_description = long_description,
    packages = find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages = ['ks',],
    install_requires = ['setuptools',
                        'zope.interface',
                        'zope.component',
                        'zope.schema',
                        'zope.publisher',
                        ],
    extras_require = dict(
        test = ['zope.testing',
                'ZODB3'],
        ),
    include_package_data = True,
    zip_safe = False,
    )
